@@ -5,7 +5,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.67 2007/10/25 15:23:09 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.75 2008/04/01 22:20:19 tg Exp $");
 
 /* A leading = means assignments before command are kept;
  * a leading * means a POSIX special builtin;
@@ -115,12 +115,12 @@ static const struct t_op b_ops[] = {
 };
 
 static int test_eaccess(const char *, int);
-static int test_oexpr(Test_env *, int);
-static int test_aexpr(Test_env *, int);
-static int test_nexpr(Test_env *, int);
-static int test_primary(Test_env *, int);
+static int test_oexpr(Test_env *, bool);
+static int test_aexpr(Test_env *, bool);
+static int test_nexpr(Test_env *, bool);
+static int test_primary(Test_env *, bool);
 static int ptest_isa(Test_env *, Test_meta);
-static const char *ptest_getopnd(Test_env *, Test_op, int);
+static const char *ptest_getopnd(Test_env *, Test_op, bool);
 static void ptest_error(Test_env *, int, const char *);
 static char *kill_fmt_entry(const void *, int, char *, int);
 static void p_time(struct shf *, int, struct timeval *, int,
@@ -129,26 +129,22 @@ static void p_time(struct shf *, int, struct timeval *, int,
 int
 c_cd(const char **wp)
 {
-	int optc;
-	int physical = Flag(FPHYSICAL);
+	int optc, rv, phys_path;
+	bool physical = Flag(FPHYSICAL) ? true : false;
 	int cdnode;			/* was a node from cdpath added in? */
-	int printpath = 0;		/* print where we cd'd? */
-	int rval;
+	bool printpath = false;		/* print where we cd'd? */
 	struct tbl *pwd_s, *oldpwd_s;
 	XString xs;
-	char *xp;
-	char *dir, *try, *pwd;
-	int phys_path;
-	char *cdpath;
+	char *dir, *try, *pwd, *cdpath;
 	bool dir_ = false;
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "LP")) != -1)
 		switch (optc) {
 		case 'L':
-			physical = 0;
+			physical = false;
 			break;
 		case 'P':
-			physical = 1;
+			physical = true;
 			break;
 		case '?':
 			return 1;
@@ -179,7 +175,7 @@ c_cd(const char **wp)
 				bi_errorf("no OLDPWD");
 				return 1;
 			}
-			printpath++;
+			printpath = true;
 		} else
 			dir_ = true;
 	} else if (!wp[2]) {
@@ -209,30 +205,26 @@ c_cd(const char **wp)
 		memcpy(dir, current_wd, ilen);
 		memcpy(dir + ilen, wp[1], nlen);
 		memcpy(dir + ilen + nlen, current_wd + ilen + olen, elen);
-		printpath++;
+		printpath = true;
 	} else {
 		bi_errorf("too many arguments");
 		return 1;
 	}
 
-	Xinit(xs, xp, PATH_MAX, ATEMP);
-	/* xp will have a bogus value after make_path() - set it to 0
-	 * so that if it's used, it will cause a dump
-	 */
-	xp = NULL;
+	XinitN(xs, PATH_MAX, ATEMP);
 
 	cdpath = str_val(global("CDPATH"));
 	do {
 		cdnode = make_path(current_wd, dir, &cdpath, &xs, &phys_path);
 		if (physical)
-			rval = chdir(try = Xstring(xs, xp) + phys_path);
+			rv = chdir(try = Xstring(xs, xp) + phys_path);
 		else {
 			simplify_path(Xstring(xs, xp));
-			rval = chdir(try = Xstring(xs, xp));
+			rv = chdir(try = Xstring(xs, xp));
 		}
-	} while (rval < 0 && cdpath != NULL);
+	} while (rv < 0 && cdpath != NULL);
 
-	if (rval < 0) {
+	if (rv < 0) {
 		if (cdnode)
 			bi_errorf("%s: bad directory", dir);
 		else
@@ -279,17 +271,17 @@ int
 c_pwd(const char **wp)
 {
 	int optc;
-	int physical = Flag(FPHYSICAL);
+	bool physical = Flag(FPHYSICAL) ? true : false;
 	char *p;
 	bool p_ = false;
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "LP")) != -1)
 		switch (optc) {
 		case 'L':
-			physical = 0;
+			physical = false;
 			break;
 		case 'P':
-			physical = 1;
+			physical = true;
 			break;
 		case '?':
 			return 1;
@@ -327,8 +319,7 @@ c_print(const char **wp)
 #define PO_COPROC	BIT(4)	/* printing to coprocess: block SIGPIPE */
 	int fd = 1;
 	int flags = PO_EXPAND|PO_NL;
-	const char *s;
-	const char *emsg;
+	const char *s, *emsg;
 	XString xs;
 	char *xp;
 
@@ -449,6 +440,57 @@ c_print(const char **wp)
 							break;
 					}
 					break;
+				case 'x':
+					/* Look for a hexadecimal number of
+					 * up to 2 digits, write raw octet.
+					 */
+					c = 0;
+					for (i = 0; i < 2; i++) {
+						c <<= 4;
+						if (*s >= '0' && *s <= '9')
+							c += *s++ - '0';
+						else if (*s >= 'A' && *s <= 'F')
+							c += *s++ - 'A' + 10;
+						else if (*s >= 'a' && *s <= 'f')
+							c += *s++ - 'a' + 10;
+						else {
+							c >>= 4;
+							break;
+						}
+					}
+					break;
+				case 'u':
+					/* Look for a hexadecimal number of
+					 * up to 4 digits, write Unicode.
+					 */
+					c = 0;
+					for (i = 0; i < 4; i++) {
+						c <<= 4;
+						if (*s >= '0' && *s <= '9')
+							c += *s++ - '0';
+						else if (*s >= 'A' && *s <= 'F')
+							c += *s++ - 'A' + 10;
+						else if (*s >= 'a' && *s <= 'f')
+							c += *s++ - 'a' + 10;
+						else {
+							c >>= 4;
+							break;
+						}
+					}
+					if (c < 0x80)
+						/* Xput below writes ASCII */;
+					else if (c < 0x0800) {
+						Xput(xs, xp, (c >> 6) | 0xC0);
+						c = 0x80 | (c & 0x3F);
+						/* leave 2nd octet to below */
+					} else {
+						Xput(xs, xp, (c >> 12) | 0xE0);
+						Xput(xs, xp,
+						    ((c >> 6) & 0x3F) | 0x80);
+						c = 0x80 | (c & 0x3F);
+						/* leave 3rd octet to below */
+					}
+					break;
 				case '\0': s--; c = '\\'; break;
 				case '\\': break;
 				default:
@@ -510,30 +552,27 @@ c_whence(const char **wp)
 {
 	struct tbl *tp;
 	const char *id;
-	int pflag = 0, vflag = 0, Vflag = 0;
-	int ret = 0;
-	int optc;
-	int iam_whence = wp[0][0] == 'w';
-	int fcflags;
+	bool pflag = false, vflag = false, Vflag = false;
+	int rv = 0, optc, fcflags;
+	bool iam_whence = wp[0][0] == 'w';
 	const char *opts = iam_whence ? "pv" : "pvV";
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, opts)) != -1)
 		switch (optc) {
 		case 'p':
-			pflag = 1;
+			pflag = true;
 			break;
 		case 'v':
-			vflag = 1;
+			vflag = true;
 			break;
 		case 'V':
-			Vflag = 1;
+			Vflag = true;
 			break;
 		case '?':
 			return 1;
 		}
 	wp += builtin_opt.optind;
 
-
 	fcflags = FC_BI | FC_PATH | FC_FUNC;
 	if (!iam_whence) {
 		/* Note that -p on its own is deal with in comexec() */
@@ -548,7 +587,7 @@ c_whence(const char **wp)
 	if (pflag)
 		fcflags &= ~(FC_BI | FC_FUNC);
 
-	while ((vflag || ret == 0) && (id = *wp++) != NULL) {
+	while ((vflag || rv == 0) && (id = *wp++) != NULL) {
 		tp = NULL;
 		if ((iam_whence || vflag) && !pflag)
 			tp = ktsearch(&keywords, id, hash(id));
@@ -561,35 +600,34 @@ c_whence(const char **wp)
 			tp = findcom(id, fcflags);
 		if (vflag || (tp->type != CALIAS && tp->type != CEXEC &&
 		    tp->type != CTALIAS))
-			shprintf("%s", id);
+			shf_puts(id, shl_stdout);
 		switch (tp->type) {
 		case CKEYWD:
 			if (vflag)
-				shprintf(" is a reserved word");
+				shf_puts(" is a reserved word", shl_stdout);
 			break;
 		case CALIAS:
 			if (vflag)
 				shprintf(" is an %salias for ",
-				    (tp->flag & EXPORT) ? "exported " :
-				    null);
+				    (tp->flag & EXPORT) ? "exported " : null);
 			if (!iam_whence && !vflag)
 				shprintf("alias %s=", id);
 			print_value_quoted(tp->val.s);
 			break;
 		case CFUNC:
 			if (vflag) {
-				shprintf(" is a");
+				shf_puts(" is a", shl_stdout);
 				if (tp->flag & EXPORT)
-					shprintf("n exported");
+					shf_puts("n exported", shl_stdout);
 				if (tp->flag & TRACE)
-					shprintf(" traced");
+					shf_puts(" traced", shl_stdout);
 				if (!(tp->flag & ISSET)) {
-					shprintf(" undefined");
+					shf_puts(" undefined", shl_stdout);
 					if (tp->u.fpath)
 						shprintf(" (autoload from %s)",
 						    tp->u.fpath);
 				}
-				shprintf(" function");
+				shf_puts(" function", shl_stdout);
 			}
 			break;
 		case CSHELL:
@@ -601,27 +639,27 @@ c_whence(const char **wp)
 		case CEXEC:
 			if (tp->flag & ISSET) {
 				if (vflag) {
-					shprintf(" is ");
+					shf_puts(" is ", shl_stdout);
 					if (tp->type == CTALIAS)
 						shprintf("a tracked %salias for ",
 						    (tp->flag & EXPORT) ?
 						    "exported " : null);
 				}
-				shprintf("%s", tp->val.s);
+				shf_puts(tp->val.s, shl_stdout);
 			} else {
 				if (vflag)
-					shprintf(" not found");
-				ret = 1;
+					shf_puts(" not found", shl_stdout);
+				rv = 1;
 			}
 			break;
 		default:
 			shprintf("%s is *GOK*", id);
 			break;
 		}
-		if (vflag || !ret)
+		if (vflag || !rv)
 			shf_putc('\n', shl_stdout);
 	}
-	return ret;
+	return rv;
 }
 
 /* Deal with command -vV - command -p dealt with in comexec() */
@@ -640,32 +678,32 @@ c_typeset(const char **wp)
 {
 	struct block *l;
 	struct tbl *vp, **p;
-	Tflag fset = 0, fclr = 0;
-	int thing = 0, func = 0, localv = 0;
-	const char *opts = "L#R#UZ#fi#lprtux";	/* see comment below */
+	Tflag fset = 0, fclr = 0, flag;
+	int thing = 0, field, base, optc;
+	const char *opts;
 	const char *fieldstr, *basestr;
-	int field, base;
-	int optc;
-	Tflag flag;
-	int pflag = 0;
+	bool localv = false, func = false, pflag = false, istset = true;
 
 	switch (**wp) {
 	case 'e':		/* export */
 		fset |= EXPORT;
-		opts = "p";
+		istset = false;
 		break;
 	case 'r':		/* readonly */
 		fset |= RDONLY;
-		opts = "p";
+		istset = false;
 		break;
 	case 's':		/* set */
 		/* called with 'typeset -' */
 		break;
 	case 't':		/* typeset */
-		localv = 1;
+		localv = true;
 		break;
 	}
 
+	/* see comment below regarding possible opions */
+	opts = istset ? "L#R#UZ#fi#lprtux" : "p";
+
 	fieldstr = basestr = NULL;
 	builtin_opt.flags |= GF_PLUSOPT;
 	/* at&t ksh seems to have 0-9 as options which are multiplied
@@ -699,7 +737,7 @@ c_typeset(const char **wp)
 			fieldstr = builtin_opt.optarg;
 			break;
 		case 'f':
-			func = 1;
+			func = true;
 			break;
 		case 'i':
 			flag = INTEGER;
@@ -709,12 +747,12 @@ c_typeset(const char **wp)
 			flag = LCASEV;
 			break;
 		case 'p':
-			/* posix export/readonly -p flag.
-			 * typeset -p is the same as typeset (in pdksh);
-			 * here for compatibility with ksh93.
-			 */
-			pflag = 1;
-			continue;
+			/* export, readonly: POSIX -p flag */
+			/* typeset: show values as well */
+			pflag = true;
+			if (istset)
+				continue;
+			break;
 		case 'r':
 			flag = RDONLY;
 			break;
@@ -784,8 +822,7 @@ c_typeset(const char **wp)
 
 	/* set variables and attributes */
 	if (wp[builtin_opt.optind]) {
-		int i;
-		int rval = 0;
+		int i, rv = 0;
 		struct tbl *f;
 
 		if (localv && !func)
@@ -795,8 +832,8 @@ c_typeset(const char **wp)
 				f = findfunc(wp[i], hash(wp[i]),
 				    (fset&UCASEV_AL) ? true : false);
 				if (!f) {
-					/* at&t ksh does ++rval: bogus */
-					rval = 1;
+					/* at&t ksh does ++rv: bogus */
+					rv = 1;
 					continue;
 				}
 				if (fset | fclr) {
@@ -812,7 +849,7 @@ c_typeset(const char **wp)
 				return 1;
 			}
 		}
-		return rval;
+		return rv;
 	}
 
 	/* list variables and attributes */
@@ -834,14 +871,14 @@ c_typeset(const char **wp)
 		for (l = e->loc; l; l = l->next) {
 			for (p = ktsort(&l->vars); (vp = *p++); ) {
 				struct tbl *tvp;
-				int any_set = 0;
+				bool any_set = false;
 				/*
 				 * See if the parameter is set (for arrays, if any
 				 * element is set).
 				 */
 				for (tvp = vp; tvp; tvp = tvp->u.array)
 					if (tvp->flag & ISSET) {
-						any_set = 1;
+						any_set = true;
 						break;
 					}
 
@@ -873,57 +910,76 @@ c_typeset(const char **wp)
 						 * but POSIX says must
 						 * be suitable for re-entry...
 						 */
-						shprintf("typeset ");
+						shf_puts("typeset ", shl_stdout);
 						if ((vp->flag&INTEGER))
-							shprintf("-i ");
+							shf_puts("-i ", shl_stdout);
 						if ((vp->flag&EXPORT))
-							shprintf("-x ");
+							shf_puts("-x ", shl_stdout);
 						if ((vp->flag&RDONLY))
-							shprintf("-r ");
+							shf_puts("-r ", shl_stdout);
 						if ((vp->flag&TRACE))
-							shprintf("-t ");
+							shf_puts("-t ", shl_stdout);
 						if ((vp->flag&LJUST))
 							shprintf("-L%d ", vp->u2.field);
 						if ((vp->flag&RJUST))
 							shprintf("-R%d ", vp->u2.field);
 						if ((vp->flag&ZEROFIL))
-							shprintf("-Z ");
+							shf_puts("-Z ", shl_stdout);
 						if ((vp->flag&LCASEV))
-							shprintf("-l ");
+							shf_puts("-l ", shl_stdout);
 						if ((vp->flag&UCASEV_AL))
-							shprintf("-u ");
+							shf_puts("-u ", shl_stdout);
 						if ((vp->flag&INT_U))
-							shprintf("-U ");
-						shprintf("%s\n", vp->name);
-						    if (vp->flag&ARRAY)
-						break;
-					} else {
-						if (pflag)
-							shprintf("typeset ");
-						if ((vp->flag&ARRAY) && any_set)
-							shprintf("%s[%lu]",
-							    vp->name,
-							    (unsigned long)vp->index);
-						else
-							shprintf("%s", vp->name);
-						if (thing == '-' && (vp->flag&ISSET)) {
+							shf_puts("-U ", shl_stdout);
+						shf_puts(vp->name, shl_stdout);
+						if (pflag) {
 							char *s = str_val(vp);
 
-							shprintf("=");
+							shf_putc('=', shl_stdout);
 							/* at&t ksh can't have
 							 * justified integers.. */
 							if ((vp->flag &
 							    (INTEGER|LJUST|RJUST)) ==
 							    INTEGER)
-								shprintf("%s", s);
+								shf_puts(s, shl_stdout);
+							else
+								print_value_quoted(s);
+						}
+						shf_putc('\n', shl_stdout);
+						if (vp->flag & ARRAY)
+							break;
+					} else {
+						if (pflag)
+							shf_puts(istset ?
+							    "typeset " :
+							    (flag & EXPORT) ?
+							    "export " :
+							    "readonly ",
+							    shl_stdout);
+						if ((vp->flag&ARRAY) && any_set)
+							shprintf("%s[%lu]",
+							    vp->name,
+							    (unsigned long)vp->index);
+						else
+							shf_puts(vp->name, shl_stdout);
+						if (thing == '-' && (vp->flag&ISSET)) {
+							char *s = str_val(vp);
+
+							shf_putc('=', shl_stdout);
+							/* at&t ksh can't have
+							 * justified integers.. */
+							if ((vp->flag &
+							    (INTEGER|LJUST|RJUST)) ==
+							    INTEGER)
+								shf_puts(s, shl_stdout);
 							else
 								print_value_quoted(s);
 						}
 						shf_putc('\n', shl_stdout);
 					}
 					/* Only report first 'element' of an array with
-					* no set elements.
-					*/
+					 * no set elements.
+					 */
 					if (!any_set)
 						break;
 				}
@@ -937,8 +993,8 @@ int
 c_alias(const char **wp)
 {
 	struct table *t = &aliases;
-	int rv = 0, rflag = 0, tflag, Uflag = 0, pflag = 0;
-	int prefix = 0;
+	int rv = 0, prefix = 0;
+	bool rflag = false, tflag, Uflag = false, pflag = false;
 	Tflag xflag = 0;
 	int optc;
 
@@ -954,10 +1010,10 @@ c_alias(const char **wp)
 #endif
 			break;
 		case 'p':
-			pflag = 1;
+			pflag = true;
 			break;
 		case 'r':
-			rflag = 1;
+			rflag = true;
 			break;
 		case 't':
 			t = &taliases;
@@ -967,7 +1023,7 @@ c_alias(const char **wp)
 			 * kludge for tracked alias initialization
 			 * (don't do a path search, just make an entry)
 			 */
-			Uflag = 1;
+			Uflag = true;
 			break;
 		case 'x':
 			xflag = EXPORT;
@@ -993,8 +1049,8 @@ c_alias(const char **wp)
 		};
 
 		if (!tflag || *wp) {
-			shprintf("alias: -r flag can only be used with -t"
-			    " and without arguments\n");
+			shf_puts("alias: -r flag can only be used with -t"
+			    " and without arguments\n", shl_stdout);
 			return 1;
 		}
 		ksh_getopt_reset(&builtin_opt, GF_ERROR);
@@ -1018,10 +1074,8 @@ c_alias(const char **wp)
 	}
 
 	for (; *wp != NULL; wp++) {
-		const char *alias = *wp;
+		const char *alias = *wp, *val, *newval;
 		char *xalias = NULL;
-		const char *val;
-		const char *newval;
 		struct tbl *ap;
 		int h;
 
@@ -1078,13 +1132,13 @@ c_unalias(const char **wp)
 {
 	struct table *t = &aliases;
 	struct tbl *ap;
-	int rv = 0, all = 0;
-	int optc;
+	int optc, rv = 0;
+	bool all = false;
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "adt")) != -1)
 		switch (optc) {
 		case 'a':
-			all = 1;
+			all = true;
 			break;
 		case 'd':
 #ifdef MKSH_SMALL
@@ -1150,10 +1204,7 @@ c_let(const char **wp)
 int
 c_jobs(const char **wp)
 {
-	int optc;
-	int flag = 0;
-	int nflag = 0;
-	int rv = 0;
+	int optc, flag = 0, nflag = 0, rv = 0;
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "lpnz")) != -1)
 		switch (optc) {
@@ -1187,7 +1238,7 @@ c_jobs(const char **wp)
 int
 c_fgbg(const char **wp)
 {
-	int bg = strcmp(*wp, "bg") == 0;
+	bool bg = strcmp(*wp, "bg") == 0;
 	int rv = 0;
 
 	if (!Flag(FMONITOR)) {
@@ -1224,7 +1275,7 @@ c_kill(const char **wp)
 {
 	Trap *t = NULL;
 	const char *p;
-	int lflag = 0;
+	bool lflag = false;
 	int i, n, rv, sig;
 
 	/* assume old style options if -digits or -UPPERCASE */
@@ -1241,7 +1292,7 @@ c_kill(const char **wp)
 		while ((optc = ksh_getopt(wp, &builtin_opt, "ls:")) != -1)
 			switch (optc) {
 			case 'l':
-				lflag = 1;
+				lflag = true;
 				break;
 			case 's':
 				if (!(t = gettrap(builtin_opt.optarg, true))) {
@@ -1335,12 +1386,9 @@ getopts_reset(int val)
 int
 c_getopts(const char **wp)
 {
-	int	argc;
-	const char *opts;
-	const char *var;
-	int	optc;
-	int	ret;
-	char	buf[3];
+	int argc, optc, rv;
+	const char *opts, *var;
+	char buf[3];
 	struct tbl *vq, *voptarg;
 
 	if (ksh_getopt(wp, &builtin_opt, null) == '?')
@@ -1417,32 +1465,33 @@ c_getopts(const char **wp)
 		/* This can't fail (have cleared readonly/integer) */
 		setstr(voptarg, user_opt.optarg, KSH_RETURN_ERROR);
 
-	ret = 0;
+	rv = 0;
 
 	vq = global(var);
 	/* Error message already printed (integer, readonly) */
 	if (!setstr(vq, buf, KSH_RETURN_ERROR))
-	    ret = 1;
+		rv = 1;
 	if (Flag(FEXPORT))
 		typeset(var, EXPORT, 0, 0, 0);
 
-	return optc < 0 ? 1 : ret;
+	return optc < 0 ? 1 : rv;
 }
 
 int
 c_bind(const char **wp)
 {
-	int optc, rv = 0, macro = 0, list = 0;
+	int optc, rv = 0;
+	bool macro = false, list = false;
 	const char *cp;
 	char *up;
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "lm")) != -1)
 		switch (optc) {
 		case 'l':
-			list = 1;
+			list = true;
 			break;
 		case 'm':
-			macro = 1;
+			macro = true;
 			break;
 		case '?':
 			return 1;
@@ -1508,16 +1557,15 @@ c_shift(const char **wp)
 int
 c_umask(const char **wp)
 {
-	int i;
+	int i, optc;
 	const char *cp;
-	int symbolic = 0;
+	bool symbolic = false;
 	mode_t old_umask;
-	int optc;
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "S")) != -1)
 		switch (optc) {
 		case 'S':
-			symbolic = 1;
+			symbolic = true;
 			break;
 		case '?':
 			return 1;
@@ -1629,28 +1677,26 @@ c_umask(const char **wp)
 		}
 		umask(new_umask);
 	}
-	return 0;
+	return (0);
 }
 
 int
 c_dot(const char **wp)
 {
-	const char *file;
-	const char *cp;
-	const char **argv;
-	int argc;
-	int i;
-	int err;
+	const char *file, *cp, **argv;
+	int argc, i, errcode;
 
 	if (ksh_getopt(wp, &builtin_opt, null) == '?')
-		return 1;
+		return (1);
 
-	if ((cp = wp[builtin_opt.optind]) == NULL)
-		return 0;
-	file = search(cp, path, R_OK, &err);
-	if (file == NULL) {
-		bi_errorf("%s: %s", cp, err ? strerror(err) : "not found");
-		return 1;
+	if ((cp = wp[builtin_opt.optind]) == NULL) {
+		bi_errorf("missing argument");
+		return (1);
+	}
+	if ((file = search(cp, path, R_OK, &errcode)) == NULL) {
+		bi_errorf("%s: %s", cp,
+		    errcode ? strerror(errcode) : "not found");
+		return (1);
 	}
 
 	/* Set positional parameters? */
@@ -1663,19 +1709,18 @@ c_dot(const char **wp)
 		argc = 0;
 		argv = NULL;
 	}
-	i = include(file, argc, argv, 0);
-	if (i < 0) { /* should not happen */
+	if ((i = include(file, argc, argv, 0)) < 0) {
+		/* should not happen */
 		bi_errorf("%s: %s", cp, strerror(errno));
-		return 1;
+		return (1);
 	}
-	return i;
+	return (i);
 }
 
 int
 c_wait(const char **wp)
 {
-	int rv = 0;
-	int sig;
+	int rv = 0, sig;
 
 	if (ksh_getopt(wp, &builtin_opt, null) == '?')
 		return 1;
@@ -1696,19 +1741,13 @@ c_wait(const char **wp)
 int
 c_read(const char **wp)
 {
-	int c = 0;
-	int expande = 1, historyr = 0;
-	int expanding;
-	int ecode = 0;
-	const char *cp;
-	char *ccp;
-	int fd = 0;
+	int c = 0, ecode = 0, fd = 0, optc;
+	bool expande = true, historyr = false, expanding;
+	const char *cp, *emsg;
 	struct shf *shf;
-	int optc;
-	const char *emsg;
 	XString cs, xs = { NULL, NULL, 0, NULL};
 	struct tbl *vp;
-	char *xp = NULL, *wpalloc = NULL;
+	char *ccp, *xp = NULL, *wpalloc = NULL;
 	static char REPLY[] = "REPLY";
 
 	while ((optc = ksh_getopt(wp, &builtin_opt, "prsu,")) != -1)
@@ -1720,10 +1759,10 @@ c_read(const char **wp)
 			}
 			break;
 		case 'r':
-			expande = 0;
+			expande = false;
 			break;
 		case 's':
-			historyr = 1;
+			historyr = true;
 			break;
 		case 'u':
 			if (!*(cp = builtin_opt.optarg))
@@ -1773,7 +1812,7 @@ c_read(const char **wp)
 
 	if (historyr)
 		Xinit(xs, xp, 128, ATEMP);
-	expanding = 0;
+	expanding = false;
 	Xinit(cs, ccp, 128, ATEMP);
 	for (; *wp != NULL; wp++) {
 		for (ccp = Xstring(cs, ccp); ; ) {
@@ -1805,7 +1844,7 @@ c_read(const char **wp)
 			}
 			Xcheck(cs, ccp);
 			if (expanding) {
-				expanding = 0;
+				expanding = false;
 				if (c == '\n') {
 					c = 0;
 					if (Flag(FTALKING_I) && isatty(fd)) {
@@ -1820,7 +1859,7 @@ c_read(const char **wp)
 				continue;
 			}
 			if (expande && c == '\\') {
-				expanding = 1;
+				expanding = true;
 				continue;
 			}
 			if (c == '\n' || c == EOF)
@@ -1935,7 +1974,7 @@ c_trap(const char **wp)
 	if (*wp == NULL) {
 		for (p = sigtraps, i = NSIG+1; --i >= 0; p++)
 			if (p->trap != NULL) {
-				shprintf("trap -- ");
+				shf_puts("trap -- ", shl_stdout);
 				print_value_quoted(p->trap);
 				shprintf(" %s\n", p->name);
 			}
@@ -1966,8 +2005,7 @@ c_trap(const char **wp)
 int
 c_exitreturn(const char **wp)
 {
-	int how = LEXIT;
-	int n;
+	int n, how = LEXIT;
 	const char *arg;
 
 	if (ksh_getopt(wp, &builtin_opt, null) == '?')
@@ -2172,10 +2210,9 @@ timex(struct op *t, int f)
 #if !defined(RUSAGE_SELF) || !defined(RUSAGE_CHILDREN)
 	return (0);
 #else
-	int rv = 0;
+	int rv = 0, tf = 0;
 	struct rusage ru0, ru1, cru0, cru1;
 	struct timeval usrtime, systime, tv0, tv1;
-	int tf = 0;
 	char opts[1];
 
 	gettimeofday(&tv0, NULL);
@@ -2239,8 +2276,7 @@ void
 timex_hook(struct op *t, char **volatile *app)
 {
 	char **wp = *app;
-	int optc;
-	int i, j;
+	int optc, i, j;
 	Getopt opt;
 
 	ksh_getopt_reset(&opt, 0);
@@ -2407,8 +2443,7 @@ c_builtin(const char **wp __unused)
 int
 c_test(const char **wp)
 {
-	int argc;
-	int res;
+	int argc, res;
 	Test_env te;
 
 	te.flags = 0;
@@ -2502,11 +2537,12 @@ test_isop(Test_meta meta, const char *s)
 
 int
 test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
-    int do_eval)
+    bool do_eval)
 {
-	int i;
+	int i, s;
 	size_t k;
 	struct stat b1, b2;
+	long v1, v2;
 
 	if (!do_eval)
 		return 0;
@@ -2602,50 +2638,40 @@ test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
 	case TO_INTGT: /* -gt */
 	case TO_INTLE: /* -le */
 	case TO_INTLT: /* -lt */
-		{
-			long v1, v2;
-
-			if (!evaluate(opnd1, &v1, KSH_RETURN_ERROR, false) ||
-			    !evaluate(opnd2, &v2, KSH_RETURN_ERROR, false)) {
-				/* error already printed.. */
-				te->flags |= TEF_ERROR;
-				return 1;
-			}
-			switch ((int) op) {
-			case TO_INTEQ:
-				return v1 == v2;
-			case TO_INTNE:
-				return v1 != v2;
-			case TO_INTGE:
-				return v1 >= v2;
-			case TO_INTGT:
-				return v1 > v2;
-			case TO_INTLE:
-				return v1 <= v2;
-			case TO_INTLT:
-				return v1 < v2;
-			}
+		if (!evaluate(opnd1, &v1, KSH_RETURN_ERROR, false) ||
+		    !evaluate(opnd2, &v2, KSH_RETURN_ERROR, false)) {
+			/* error already printed.. */
+			te->flags |= TEF_ERROR;
+			return 1;
+		}
+		switch ((int)op) {
+		case TO_INTEQ:
+			return (v1 == v2);
+		case TO_INTNE:
+			return (v1 != v2);
+		case TO_INTGE:
+			return (v1 >= v2);
+		case TO_INTGT:
+			return (v1 > v2);
+		case TO_INTLE:
+			return (v1 <= v2);
+		case TO_INTLT:
+			return (v1 < v2);
 		}
 	case TO_FILNT: /* -nt */
-		{
-			int s2;
-			/* ksh88/ksh93 succeed if file2 can't be stated
-			 * (subtly different from 'does not exist').
-			 */
-			return stat(opnd1, &b1) == 0 &&
-			    (((s2 = stat(opnd2, &b2)) == 0 &&
-			    b1.st_mtime > b2.st_mtime) || s2 < 0);
-		}
+		/* ksh88/ksh93 succeed if file2 can't be stated
+		 * (subtly different from 'does not exist').
+		 */
+		return stat(opnd1, &b1) == 0 &&
+		    (((s = stat(opnd2, &b2)) == 0 &&
+		    b1.st_mtime > b2.st_mtime) || s < 0);
 	case TO_FILOT: /* -ot */
-		{
-			int s1;
-			/* ksh88/ksh93 succeed if file1 can't be stated
-			 * (subtly different from 'does not exist').
-			 */
-			return stat(opnd2, &b2) == 0 &&
-			    (((s1 = stat(opnd1, &b1)) == 0 &&
-			    b1.st_mtime < b2.st_mtime) || s1 < 0);
-		}
+		/* ksh88/ksh93 succeed if file1 can't be stated
+		 * (subtly different from 'does not exist').
+		 */
+		return stat(opnd2, &b2) == 0 &&
+		    (((s = stat(opnd1, &b1)) == 0 &&
+		    b1.st_mtime < b2.st_mtime) || s < 0);
 	case TO_FILEQ: /* -ef */
 		return stat (opnd1, &b1) == 0 && stat (opnd2, &b2) == 0 &&
 		    b1.st_dev == b2.st_dev && b1.st_ino == b2.st_ino;
@@ -2658,63 +2684,61 @@ test_eval(Test_env *te, Test_op op, const char *opnd1, const char *opnd2,
 static int
 test_eaccess(const char *pathl, int mode)
 {
-	int res = access(pathl, mode);
+	int rv;
 
-	if (res == 0 && ksheuid == 0 && (mode & X_OK)) {
+	if ((rv = access(pathl, mode)) == 0 && ksheuid == 0 && (mode & X_OK)) {
 		struct stat statb;
 
 		if (stat(pathl, &statb) < 0)
-			res = -1;
+			rv = -1;
 		else if (S_ISDIR(statb.st_mode))
-			res = 0;
+			rv = 0;
 		else
-			res = (statb.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH)) ?
+			rv = (statb.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH)) ?
 			    0 : -1;
 	}
-	return res;
+	return rv;
 }
 
 int
 test_parse(Test_env *te)
 {
-	int res;
+	int rv;
 
-	res = test_oexpr(te, 1);
+	rv = test_oexpr(te, 1);
 
 	if (!(te->flags & TEF_ERROR) && !(*te->isa)(te, TM_END))
 		(*te->error)(te, 0, "unexpected operator/operand");
 
-	return (te->flags & TEF_ERROR) ? T_ERR_EXIT : !res;
+	return (te->flags & TEF_ERROR) ? T_ERR_EXIT : !rv;
 }
 
 static int
-test_oexpr(Test_env *te, int do_eval)
+test_oexpr(Test_env *te, bool do_eval)
 {
-	int res;
+	int rv;
 
-	res = test_aexpr(te, do_eval);
-	if (res)
-		do_eval = 0;
+	if ((rv = test_aexpr(te, do_eval)))
+		do_eval = false;
 	if (!(te->flags & TEF_ERROR) && (*te->isa)(te, TM_OR))
-		return test_oexpr(te, do_eval) || res;
-	return res;
+		return test_oexpr(te, do_eval) || rv;
+	return rv;
 }
 
 static int
-test_aexpr(Test_env *te, int do_eval)
+test_aexpr(Test_env *te, bool do_eval)
 {
-	int res;
+	int rv;
 
-	res = test_nexpr(te, do_eval);
-	if (!res)
-		do_eval = 0;
+	if (!(rv = test_nexpr(te, do_eval)))
+		do_eval = false;
 	if (!(te->flags & TEF_ERROR) && (*te->isa)(te, TM_AND))
-		return test_aexpr(te, do_eval) && res;
-	return res;
+		return test_aexpr(te, do_eval) && rv;
+	return rv;
 }
 
 static int
-test_nexpr(Test_env *te, int do_eval)
+test_nexpr(Test_env *te, bool do_eval)
 {
 	if (!(te->flags & TEF_ERROR) && (*te->isa)(te, TM_NOT))
 		return !test_nexpr(te, do_eval);
@@ -2722,23 +2746,23 @@ test_nexpr(Test_env *te, int do_eval)
 }
 
 static int
-test_primary(Test_env *te, int do_eval)
+test_primary(Test_env *te, bool do_eval)
 {
 	const char *opnd1, *opnd2;
-	int res;
+	int rv;
 	Test_op op;
 
 	if (te->flags & TEF_ERROR)
 		return 0;
 	if ((*te->isa)(te, TM_OPAREN)) {
-		res = test_oexpr(te, do_eval);
+		rv = test_oexpr(te, do_eval);
 		if (te->flags & TEF_ERROR)
 			return 0;
 		if (!(*te->isa)(te, TM_CPAREN)) {
 			(*te->error)(te, 0, "missing closing paren");
 			return 0;
 		}
-		return res;
+		return rv;
 	}
 	if ((op = (*te->isa)(te, TM_UNOP))) {
 		/* unary expression */
@@ -2787,27 +2811,27 @@ ptest_isa(Test_env *te, Test_meta meta)
 	static const char *const tokens[] = {
 		"-o", "-a", "!", "(", ")"
 	};
-	int ret;
+	int rv;
 
 	if (te->pos.wp >= te->wp_end)
 		return meta == TM_END;
 
 	if (meta == TM_UNOP || meta == TM_BINOP)
-		ret = test_isop(meta, *te->pos.wp);
+		rv = test_isop(meta, *te->pos.wp);
 	else if (meta == TM_END)
-		ret = 0;
+		rv = 0;
 	else
-		ret = strcmp(*te->pos.wp, tokens[(int) meta]) == 0;
+		rv = strcmp(*te->pos.wp, tokens[(int) meta]) == 0;
 
 	/* Accept the token? */
-	if (ret)
+	if (rv)
 		te->pos.wp++;
 
-	return ret;
+	return rv;
 }
 
 static const char *
-ptest_getopnd(Test_env *te, Test_op op, int do_eval __unused)
+ptest_getopnd(Test_env *te, Test_op op, bool do_eval __unused)
 {
 	if (te->pos.wp >= te->wp_end)
 		return op == TO_FILTT ? "1" : NULL;
@@ -2815,13 +2839,12 @@ ptest_getopnd(Test_env *te, Test_op op, int do_eval __unused)
 }
 
 static void
-ptest_error(Test_env *te, int offset, const char *msg)
+ptest_error(Test_env *te, int ofs, const char *msg)
 {
-	const char *op = te->pos.wp + offset >= te->wp_end ?
-	    NULL : te->pos.wp[offset];
+	const char *op;
 
 	te->flags |= TEF_ERROR;
-	if (op)
+	if ((op = te->pos.wp + ofs >= te->wp_end ? NULL : te->pos.wp[ofs]))
 		bi_errorf("%s: %s", op, msg);
 	else
 		bi_errorf("%s", msg);
@@ -2894,15 +2917,15 @@ c_ulimit(const char **wp)
 		{ "swap(KiB)", RLIMIT, RLIMIT_SWAP, RLIMIT_SWAP,
 		    1024, 'w' },
 #endif
-		{ NULL, 0, 0, 0, 0, 0 }
+		{ NULL, RLIMIT, 0, 0, 0, 0 }
 	};
-	static char	opts[3 + NELEM(limits)];
-	rlim_t		val = (rlim_t)0;
-	int		how = SOFT | HARD;
-	const struct limits	*l;
-	int		set, all = 0;
-	int		optc, what;
-	struct rlimit	limit;
+	static char opts[3 + NELEM(limits)];
+	rlim_t val = (rlim_t)0;
+	int how = SOFT | HARD, optc, what;
+	bool all = false, set;
+	const struct limits *l;
+	struct rlimit limit;
+
 	if (!opts[0]) {
 		/* build options string on first call - yuck */
 		char *p = opts;
@@ -2922,7 +2945,7 @@ c_ulimit(const char **wp)
 			how = SOFT;
 			break;
 		case 'a':
-			all = 1;
+			all = true;
 			break;
 		case '?':
 			return 1;
@@ -2938,8 +2961,7 @@ c_ulimit(const char **wp)
 	}
 
 	wp += builtin_opt.optind;
-	set = *wp ? 1 : 0;
-	if (set) {
+	if ((set = *wp ? true : false)) {
 		if (all || wp[1]) {
 			bi_errorf("too many arguments");
 			return 1;
@@ -2985,7 +3007,7 @@ c_ulimit(const char **wp)
 			}
 			shprintf("%-20s ", l->name);
 			if (val == (rlim_t)RLIM_INFINITY)
-				shprintf("unlimited\n");
+				shf_puts("unlimited\n", shl_stdout);
 			else {
 				val = (rlim_t)(val / l->factor);
 				shprintf("%ld\n", (long)val);
@@ -3018,7 +3040,7 @@ c_ulimit(const char **wp)
 	}
 	if (!set) {
 		if (val == (rlim_t)RLIM_INFINITY)
-			shprintf("unlimited\n");
+			shf_puts("unlimited\n", shl_stdout);
 		else {
 			val = (rlim_t)(val / l->factor);
 			shprintf("%ld\n", (long)val);
