@@ -3,7 +3,15 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.57 2007/10/25 15:34:30 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.60 2008/04/02 16:55:06 tg Exp $");
+
+/*-
+ * MirOS: This is the default mapping type, and need not be specified.
+ * IRIX doesn’t have this constant.
+ */
+#ifndef MAP_FILE
+#define MAP_FILE	0
+#endif
 
 Trap sigtraps[NSIG + 1];
 static struct sigaction Sigact_ign;
@@ -614,9 +622,9 @@ void
 hist_init(Source *s)
 {
 #if HAVE_PERSISTENT_HISTORY
-	unsigned char	*base;
-	int	lines;
-	int	fd;
+	unsigned char *base;
+	int lines;
+	int fd;
 #endif
 
 	if (Flag(FTALKING) == 0)
@@ -655,13 +663,14 @@ hist_init(Source *s)
 		/*
 		 * we have some data
 		 */
-		base = mmap(NULL, hsize, PROT_READ, MAP_FILE | MAP_PRIVATE,
-		    histfd, 0);
+		base = (void *)mmap(NULL, hsize, PROT_READ,
+		    MAP_FILE | MAP_PRIVATE, histfd, 0);
 		/*
 		 * check on its validity
 		 */
-		if (base == MAP_FAILED || *base != HMAGIC1 || base[1] != HMAGIC2) {
-			if (base != MAP_FAILED)
+		if (base == (unsigned char *)MAP_FAILED ||
+		    *base != HMAGIC1 || base[1] != HMAGIC2) {
+			if (base != (unsigned char *)MAP_FAILED)
 				munmap((caddr_t)base, hsize);
 			hist_finish();
 			unlink(hname);
@@ -884,9 +893,9 @@ writehistfile(int lno, char *cmd)
 		if (sizenow > hsize) {
 			/* someone has added some lines */
 			bytes = sizenow - hsize;
-			base = mmap(NULL, sizenow, PROT_READ,
+			base = (void *)mmap(NULL, sizenow, PROT_READ,
 			    MAP_FILE | MAP_PRIVATE, histfd, 0);
-			if (base == MAP_FAILED)
+			if (base == (unsigned char *)MAP_FAILED)
 				goto bad;
 			new = base + hsize;
 			if (*new != COMMAND) {
