@@ -13,7 +13,7 @@
 #include <locale.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/main.c,v 1.89 2007/10/25 15:23:10 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/main.c,v 1.95 2008/04/01 20:40:21 tg Exp $");
 
 extern char **environ;
 
@@ -35,10 +35,6 @@ static const char *initcoms[] = {
 	"typeset", "-i", "PPID", "OPTIND=1", NULL,
 	"eval", "typeset -i RANDOM SECONDS=\"${SECONDS-0}\" TMOUT=\"${TMOUT-0}\"", NULL,
 	"alias", "integer=typeset -i", "local=typeset", NULL,
-	NULL
-};
-
-static const char *initcoms_korn[] = {
 	"alias",
 	"hash=alias -t",	/* not "alias -t --": hash -r needs to work */
 	"type=whence -v",
@@ -68,7 +64,7 @@ main(int argc, const char *argv[])
 	Source *s;
 	struct block *l;
 	int restricted, errexit;
-	const char **wp, *cc;
+	const char **wp;
 	struct env env;
 	pid_t ppid;
 	struct tbl *vp;
@@ -157,15 +153,6 @@ main(int argc, const char *argv[])
 	/* setstr can't fail here */
 	setstr(vp, def_path, KSH_RETURN_ERROR);
 
-	/* Set FPOSIX if we're called as -sh or /bin/sh or so */
-	cc = kshname;
-	i = 0; argi = 0;
-	while (cc[i] != '\0')
-		if ((cc[i++] | 2) == '/')
-			argi = i;
-	if (((cc[argi] | 0x20) == 's') && ((cc[argi + 1] | 0x20) == 'h'))
-		Flag(FPOSIX) = 1;
-
 	/* Turn on nohup by default for now - will change to off
 	 * by default once people are aware of its existence
 	 * (at&t ksh does not have a nohup option - it always sends
@@ -256,13 +243,6 @@ main(int argc, const char *argv[])
 	if (argi < 0)
 		exit(1);
 
-	if (!Flag(FPOSIX))
-		for (wp = initcoms_korn; *wp != NULL; wp++) {
-			shcomexec(wp);
-			while (*wp != NULL)
-				wp++;
-		}
-
 	if (Flag(FCOMMAND)) {
 		s = pushs(SSTRING, ATEMP);
 		if (!(s->start = s->str = argv[argi++]))
@@ -310,12 +290,14 @@ main(int argc, const char *argv[])
 		    (stristr((x), "UTF-8") || stristr((x), "utf8")))
 		/* Check if we're in a UTF-8 locale */
 		if (!Flag(FUTFHACK)) {
-			cc = setlocale(LC_CTYPE, "");
+			const char *ccp;
+
+			ccp = setlocale(LC_CTYPE, "");
 #if HAVE_LANGINFO_CODESET
-			if (!isuc(cc))
-				cc = nl_langinfo(CODESET);
+			if (!isuc(ccp))
+				ccp = nl_langinfo(CODESET);
 #endif
-			Flag(FUTFHACK) = isuc(cc);
+			Flag(FUTFHACK) = isuc(ccp);
 		}
 #undef isuc
 #endif
@@ -940,14 +922,19 @@ initio(void)
 
 /* A dup2() with error checking */
 int
-ksh_dup2(int ofd, int nfd, int errok)
+ksh_dup2(int ofd, int nfd, bool errok)
 {
-	int ret = dup2(ofd, nfd);
+	int rv;
 
-	if (ret < 0 && errno != EBADF && !errok)
+	if (((rv = dup2(ofd, nfd)) < 0) && !errok && (errno != EBADF))
 		errorf("too many files open in shell");
 
-	return ret;
+#ifdef __ultrix
+	if (rv >= 0)
+		fcntl(nfd, F_SETFD, 0);
+#endif
+
+	return (rv);
 }
 
 /*
@@ -1131,13 +1118,31 @@ maketemp(Area *ap, Temp_type type, struct temp **tlist)
 	const char *dir;
 
 	dir = tmpdir ? tmpdir : "/tmp";
+#if HAVE_MKSTEMP
 	len = strlen(dir) + 6 + 10 + 1;
+#else
+	pathname = tempnam(dir, "mksh.");
+	len = ((pathname == NULL) ? 0 : strlen(pathname)) + 1;
+#endif
 	tp = (struct temp *) alloc(sizeof(struct temp) + len, ap);
-	tp->name = pathname = (char *)&tp[1];
+	tp->name = (char *)&tp[1];
+#if !HAVE_MKSTEMP
+	if (pathname == NULL)
+		tp->name[0] = '\0';
+	else {
+		memcpy(tp->name, pathname, len);
+		free(pathname);
+	}
+#endif
+	pathname = tp->name;
 	tp->shf = NULL;
 	tp->type = type;
+#if HAVE_MKSTEMP
 	shf_snprintf(pathname, len, "%s/mksh.XXXXXXXXXX", dir);
 	if ((fd = mkstemp(pathname)) >= 0)
+#else
+	if (tp->name[0] && (fd = open(tp->name, O_CREAT | O_RDWR, 0600)) >= 0)
+#endif
 		tp->shf = shf_fdopen(fd, SHF_WR, NULL);
 	tp->pid = procpid;
 
