@@ -1,4 +1,4 @@
-/**	$MirOS: src/bin/mksh/setmode.c,v 1.9 2007/10/25 15:23:10 tg Exp $ */
+/**	$MirOS: src/bin/mksh/setmode.c,v 1.10 2008/04/06 23:27:19 tg Exp $ */
 /**	$miros: src/lib/libc/gen/setmode.c,v 1.9 2007/10/25 15:13:39 tg Exp $ */
 /*	$OpenBSD: setmode.c,v 1.17 2005/08/08 08:05:34 espie Exp $	*/
 /*	$NetBSD: setmode.c,v 1.15 1997/02/07 22:21:06 christos Exp $	*/
@@ -58,7 +58,8 @@
 #endif
 
 __SCCSID("@(#)setmode.c	8.2 (Berkeley) 3/25/94");
-__RCSID("$MirOS: src/bin/mksh/setmode.c,v 1.9 2007/10/25 15:23:10 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/setmode.c,v 1.10 2008/04/06 23:27:19 tg Exp $");
+__RCSID("$miros: src/lib/libc/gen/setmode.c,v 1.9 2007/10/25 15:13:39 tg Exp $");
 
 /* for mksh */
 #ifdef ksh_isdigit
@@ -66,6 +67,12 @@ __RCSID("$MirOS: src/bin/mksh/setmode.c,v 1.9 2007/10/25 15:23:10 tg Exp $");
 #define isdigit ksh_isdigit
 #endif
 
+/* for portability */
+#ifndef S_ISTXT
+#define S_ISTXT		0001000
+#endif
+
+
 #define	SET_LEN	6		/* initial # of bitcmd struct to malloc */
 #define	SET_LEN_INCR 4		/* # of bitcmd structs to add as needed */
 
