@@ -8,8 +8,8 @@
 /*	$OpenBSD: c_test.h,v 1.4 2004/12/20 11:34:26 otto Exp $	*/
 /*	$OpenBSD: tty.h,v 1.5 2004/12/20 11:34:26 otto Exp $	*/
 
-#define MKSH_SH_H_ID "$MirOS: src/bin/mksh/sh.h,v 1.184 2007/10/25 15:34:57 tg Exp $"
-#define MKSH_VERSION "R32 2007/10/25"
+#define MKSH_SH_H_ID "$MirOS: src/bin/mksh/sh.h,v 1.204 2008/04/11 19:55:24 tg Exp $"
+#define MKSH_VERSION "R33 2008/04/11"
 
 #if HAVE_SYS_PARAM_H
 #include <sys/param.h>
@@ -164,8 +164,8 @@ typedef int bool;
 #define ksh_isspace(c)	ksh_isspace_((unsigned)(c))
 #endif
 
-#ifndef S_ISTXT
-#define S_ISTXT		0001000
+#ifndef S_ISLNK
+#define S_ISLNK(m)	((m & 0170000) == 0120000)
 #endif
 #ifndef S_ISSOCK
 #define S_ISSOCK(m)	((m & 0170000) == 0140000)
@@ -178,12 +178,20 @@ typedef int bool;
 #define RLIMIT_VMEM	RLIMIT_AS
 #endif
 
-#if !defined(MAP_FAILED) && defined(__linux)
+#if !defined(MAP_FAILED)
+#  if defined(__linux)
 #define MAP_FAILED	((void *)-1)
+#  elif defined(__bsdi__) || defined(__ultrix)
+#define MAP_FAILED	((caddr_t)-1)
+#  endif
 #endif
 
 #ifndef NSIG
+#if defined(_NSIG)
 #define NSIG		_NSIG
+#elif defined(SIGMAX)
+#define NSIG		(SIGMAX+1)
+#endif
 #endif
 
 #undef BAD		/* AIX defines that somewhere */
@@ -200,11 +208,23 @@ extern void arc4random_addrandom(unsigned char *, int)
 extern uint32_t arc4random_pushb(void *, size_t);
 #endif
 
+#if !HAVE_FLOCK_DECL
+extern int flock(int, int);
+#endif
+
+#if !HAVE_REVOKE_DECL
+extern int revoke(const char *);
+#endif
+
 #if !HAVE_SETMODE
 mode_t getmode(const void *, mode_t);
 void *setmode(const char *);
 #endif
 
+#ifdef __ultrix
+int strcasecmp(const char *, const char *);
+#endif
+
 #if !HAVE_STRCASESTR
 const char *stristr(const char *, const char *);
 #endif
@@ -995,33 +1015,36 @@ typedef struct XString {
 typedef char *XStringP;
 
 /* initialise expandable string */
-#define Xinit(xs, xp, length, area) do { \
-			(xs).len = length; \
-			(xs).areap = (area); \
-			(xs).beg = alloc((xs).len + X_EXTRA, (xs).areap); \
-			(xs).end = (xs).beg + (xs).len; \
-			xp = (xs).beg; \
-		} while (0)
+#define XinitN(xs, length, area) do {				\
+	(xs).len = (length);					\
+	(xs).areap = (area);					\
+	(xs).beg = alloc((xs).len + X_EXTRA, (xs).areap);	\
+	(xs).end = (xs).beg + (xs).len;				\
+} while (0)
+#define Xinit(xs, xp, length, area) do {			\
+	XinitN((xs), (length), (area));				\
+	(xp) = (xs).beg;					\
+} while (0)
 
 /* stuff char into string */
 #define Xput(xs, xp, c)	(*xp++ = (c))
 
 /* check if there are at least n bytes left */
-#define XcheckN(xs, xp, n) do { \
-			int more = ((xp) + (n)) - (xs).end; \
-			if (more > 0) \
-				xp = Xcheck_grow_(&xs, xp, more); \
-		} while (0)
+#define XcheckN(xs, xp, n) do {					\
+	int more = ((xp) + (n)) - (xs).end;			\
+	if (more > 0)						\
+		(xp) = Xcheck_grow_(&(xs), (xp), more);		\
+} while (0)
 
 /* check for overflow, expand string */
-#define Xcheck(xs, xp)	XcheckN(xs, xp, 1)
+#define Xcheck(xs, xp)	XcheckN((xs), (xp), 1)
 
 /* free string */
-#define Xfree(xs, xp)	afree((void*) (xs).beg, (xs).areap)
+#define Xfree(xs, xp)	afree((void*)(xs).beg, (xs).areap)
 
 /* close, return string */
-#define Xclose(xs, xp)	(char*) aresize((void*)(xs).beg, \
-					(size_t)((xp) - (xs).beg), (xs).areap)
+#define Xclose(xs, xp)	(char*)aresize((void*)(xs).beg, \
+			    (size_t)((xp) - (xs).beg), (xs).areap)
 /* begin of string */
 #define Xstring(xs, xp)	((xs).beg)
 
@@ -1042,31 +1065,31 @@ typedef struct XPtrV {
 	void **beg, **end;	/* begin, end of vector */
 } XPtrV;
 
-#define XPinit(x, n) do { \
-			void **vp__; \
-			vp__ = (void**) alloc(sizeofN(void*, n), ATEMP); \
-			(x).cur = (x).beg = vp__; \
-			(x).end = vp__ + n; \
-		} while (0)
+#define XPinit(x, n) do {					\
+	void **vp__;						\
+	vp__ = (void**) alloc(sizeofN(void*, (n)), ATEMP);	\
+	(x).cur = (x).beg = vp__;				\
+	(x).end = vp__ + (n);					\
+} while (0)
 
-#define XPput(x, p) do { \
-			if ((x).cur >= (x).end) { \
-				int n = XPsize(x); \
-				(x).beg = (void**) aresize((void*) (x).beg, \
-				    sizeofN(void*, n*2), ATEMP); \
-				(x).cur = (x).beg + n; \
-				(x).end = (x).cur + n; \
-			} \
-			*(x).cur++ = (p); \
-		} while (0)
+#define XPput(x, p) do {					\
+	if ((x).cur >= (x).end) {				\
+		int n = XPsize(x);				\
+		(x).beg = (void**) aresize((void*) (x).beg,	\
+		    sizeofN(void *, n * 2), ATEMP);		\
+		(x).cur = (x).beg + n;				\
+		(x).end = (x).cur + n;				\
+	}							\
+	*(x).cur++ = (p);					\
+} while (0)
 
 #define XPptrv(x)	((x).beg)
 #define XPsize(x)	((x).cur - (x).beg)
 
-#define XPclose(x)	(void**) aresize((void*)(x).beg, \
-					 sizeofN(void*, XPsize(x)), ATEMP)
+#define XPclose(x)	(void**)aresize((void*)(x).beg, \
+			    sizeofN(void *, XPsize(x)), ATEMP)
 
-#define XPfree(x)	afree((void*) (x).beg, ATEMP)
+#define XPfree(x)	afree((void *)(x).beg, ATEMP)
 
 #define IDENT	64
 
@@ -1110,25 +1133,6 @@ struct source {
 #define SF_TTY		BIT(3)	/* type == SSTDIN & it is a tty */
 #define SF_FIRST	BIT(4)	/* initial state (to ignore UTF-8 BOM) */
 
-/*
- * states while lexing word
- */
-#define SBASE		0	/* outside any lexical constructs */
-#define SWORD		1	/* implicit quoting for substitute() */
-#define SLETPAREN	2	/* inside (( )), implicit quoting */
-#define SSQUOTE		3	/* inside '' */
-#define SDQUOTE		4	/* inside "" */
-#define SBRACE		5	/* inside ${} */
-#define SCSPAREN	6	/* inside $() */
-#define SBQUOTE		7	/* inside `` */
-#define SASPAREN	8	/* inside $(( )) */
-#define SHEREDELIM	9	/* parsing <<,<<- delimiter */
-#define SHEREDQUOTE	10	/* parsing " in <<,<<- delimiter */
-#define SPATTERN	11	/* parsing *(...|...) pattern (*+?@!) */
-#define STBRACE		12	/* parsing ${..[#%]..} */
-#define SLETARRAY	13	/* inside =( ), just copy */
-#define SADELIM		14	/* like SBASE, looking for delimiter */
-
 typedef union {
 	int i;
 	char *cp;
@@ -1377,7 +1381,7 @@ void shprintf(const char *, ...)
     __attribute__((format (printf, 1, 2)));
 int can_seek(int);
 void initio(void);
-int ksh_dup2(int, int, int);
+int ksh_dup2(int, int, bool);
 short savefd(int);
 void restfd(int, int);
 void openpipe(int *);
@@ -1413,7 +1417,7 @@ void change_flag(enum sh_flag, int, char);
 int parse_args(const char **, int, int *);
 int getn(const char *, int *);
 int bi_getn(const char *, int *);
-int gmatchx(const char *, const char *, int);
+int gmatchx(const char *, const char *, bool);
 int has_globbing(const char *, const char *);
 const unsigned char *pat_scan(const unsigned char *, const unsigned char *, int);
 int xstrcmp(const void *, const void *);
@@ -1469,7 +1473,7 @@ char *snptreef(char *, int, const char *, ...);
 struct op *tcopy(struct op *, Area *);
 char *wdcopy(const char *, Area *);
 const char *wdscan(const char *, int);
-char *wdstrip(const char *);
+char *wdstrip(const char *, bool, bool);
 void tfree(struct op *, Area *);
 /* var.c */
 void newblock(void);
@@ -1535,15 +1539,15 @@ struct test_env {
 	} pos;
 	const char **wp_end;	/* used by ptest_* */
 	int (*isa)(Test_env *, Test_meta);
-	const char *(*getopnd) (Test_env *, Test_op, int);
-	int (*eval)(Test_env *, Test_op, const char *, const char *, int);
+	const char *(*getopnd) (Test_env *, Test_op, bool);
+	int (*eval)(Test_env *, Test_op, const char *, const char *, bool);
 	void (*error)(Test_env *, int, const char *);
 };
 
 extern const char *const dbtest_tokens[];
 
 Test_op	test_isop(Test_meta, const char *);
-int test_eval(Test_env *, Test_op, const char *, const char *, int);
+int test_eval(Test_env *, Test_op, const char *, const char *, bool);
 int test_parse(Test_env *);
 
 EXTERN int tty_fd I__(-1);	/* dup'd tty file descriptor */
